@@ -1,5 +1,5 @@
 <?php
-# MantisBT - a php based bugtracking system
+# MantisBT - A PHP based bugtracking system
 
 # MantisBT is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,70 +15,60 @@
 # along with MantisBT.  If not, see <http://www.gnu.org/licenses/>.
 
 /**
- * Access Api
+ * Access API
  *
- * @copyright Copyright (C) 2000 - 2002  Kenzaburo Ito - kenito@300baud.org
- * @copyright Copyright (C) 2002 - 2014  MantisBT Team - mantisbt-dev@lists.sourceforge.net
- * @link http://www.mantisbt.org
  * @package CoreAPI
  * @subpackage AccessAPI
+ * @copyright Copyright 2000 - 2002  Kenzaburo Ito - kenito@300baud.org
+ * @copyright Copyright 2002  MantisBT Team - mantisbt-dev@lists.sourceforge.net
+ * @link http://www.mantisbt.org
  *
- * @uses config_api.php
- * @uses auth_api.php
+ * @uses authentication_api.php
  * @uses bug_api.php
  * @uses bugnote_api.php
- * @uses project_api.php
- * @uses helper_api.php
+ * @uses config_api.php
+ * @uses constant_inc.php
+ * @uses current_user_api.php
  * @uses database_api.php
+ * @uses error_api.php
+ * @uses helper_api.php
+ * @uses lang_api.php
+ * @uses print_api.php
+ * @uses project_api.php
+ * @uses string_api.php
+ * @uses user_api.php
  */
 
-/**
- * require constaint_inc.php for NOBODY etc
- */
-require_once( 'constant_inc.php' );
-/**
- * requires helper_api
- */
-require_once( 'helper_api.php' );
-/**
- * requires authentication_api
- */
-require_once( 'authentication_api.php' );
-/**
- * requires user_api
- */
-require_once( 'user_api.php' );
-/**
- * requires bug_api
- */
-require_once( 'bug_api.php' );
-/**
- * requires project_api
- */
-require_once( 'project_api.php' );
+require_api( 'authentication_api.php' );
+require_api( 'bug_api.php' );
+require_api( 'bugnote_api.php' );
+require_api( 'config_api.php' );
+require_api( 'constant_inc.php' );
+require_api( 'current_user_api.php' );
+require_api( 'database_api.php' );
+require_api( 'error_api.php' );
+require_api( 'helper_api.php' );
+require_api( 'lang_api.php' );
+require_api( 'print_api.php' );
+require_api( 'project_api.php' );
+require_api( 'string_api.php' );
+require_api( 'user_api.php' );
 
-/**
- *
- * @global array $g_cache_access_matrix
- */
+# @global array $g_cache_access_matrix
 $g_cache_access_matrix = array();
 
-/**
- *
- * @global array $g_cache_access_matrix_project_ids
- */
+# @global array $g_cache_access_matrix_project_ids
 $g_cache_access_matrix_project_ids = array();
 
-/**
- *
- * @global array $g_cache_access_matrix_user_ids
- */
+# @global array $g_cache_access_matrix_user_ids
 $g_cache_access_matrix_user_ids = array();
 
 /**
  * Function to be called when a user is attempting to access a page that
  * he/she is not authorised to.  This outputs an access denied message then
  * re-directs to the mainpage.
+ *
+ * @return void
  */
 function access_denied() {
 	if( !auth_is_user_authenticated() ) {
@@ -88,7 +78,7 @@ function access_denied() {
 				$t_return_page .= '?' . $_SERVER['QUERY_STRING'];
 			}
 			$t_return_page = string_url( string_sanitize_url( $t_return_page ) );
-			print_header_redirect( 'login_page.php' . '?return=' . $t_return_page );
+			print_header_redirect( 'login_page.php?return=' . $t_return_page );
 		}
 	} else {
 		if( current_user_is_anonymous() ) {
@@ -98,19 +88,23 @@ function access_denied() {
 					$t_return_page .= '?' . $_SERVER['QUERY_STRING'];
 				}
 				$t_return_page = string_url( string_sanitize_url( $t_return_page ) );
-				echo '<center>';
-				echo '<p>' . error_string( ERROR_ACCESS_DENIED ) . '</p>';
+				echo '<p class="center">' . error_string( ERROR_ACCESS_DENIED ) . '</p><p class="center">';
 				print_bracket_link( helper_mantis_url( 'login_page.php' ) . '?return=' . $t_return_page, lang_get( 'click_to_login' ) );
-				echo '<p></p>';
-				print_bracket_link( helper_mantis_url( 'main_page.php' ), lang_get( 'proceed' ) );
-
-				echo '</center>';
+				echo '</p><p class="center">';
+				print_bracket_link(
+					helper_mantis_url( config_get( 'default_home_page' ) ),
+					lang_get( 'proceed' )
+				);
+				echo '</p>';
 			}
 		} else {
-			echo '<center>';
-			echo '<p>' . error_string( ERROR_ACCESS_DENIED ) . '</p>';
-			print_bracket_link( helper_mantis_url( 'main_page.php' ), lang_get( 'proceed' ) );
-			echo '</center>';
+			echo '<p class="center">' . error_string( ERROR_ACCESS_DENIED ) . '</p>';
+			echo '<p class="center">';
+			print_bracket_link(
+				helper_mantis_url( config_get( 'default_home_page' ) ),
+				lang_get( 'proceed' )
+			);
+			echo '</p>';
 		}
 	}
 	exit;
@@ -118,8 +112,8 @@ function access_denied() {
 
 /**
  * retrieves and returns access matrix for a project from cache or caching if required.
- * @param int $p_project_id integer representing project id
- * @return  array returns an array of users->accesslevel for the given user
+ * @param integer $p_project_id Integer representing project identifier.
+ * @return array returns an array of users->accesslevel for the given user
  * @access private
  */
 function access_cache_matrix_project( $p_project_id ) {
@@ -129,28 +123,22 @@ function access_cache_matrix_project( $p_project_id ) {
 		return array();
 	}
 
-	if( !in_array( (int) $p_project_id, $g_cache_access_matrix_project_ids ) ) {
-		$t_project_user_list_table = db_get_table( 'mantis_project_user_list_table' );
-
-		$query = "SELECT user_id, access_level
-					  FROM $t_project_user_list_table
-					  WHERE project_id=" . db_param();
-		$result = db_query_bound( $query, Array( (int)$p_project_id ) );
-		$count = db_num_rows( $result );
-		for( $i = 0;$i < $count;$i++ ) {
-			$row = db_fetch_array( $result );
-
-			$g_cache_access_matrix[(int) $row['user_id']][(int) $p_project_id] = (int) $row['access_level'];
+	if( !in_array( (int)$p_project_id, $g_cache_access_matrix_project_ids ) ) {
+		db_param_push();
+		$t_query = 'SELECT user_id, access_level FROM {project_user_list} WHERE project_id=' . db_param();
+		$t_result = db_query( $t_query, array( (int)$p_project_id ) );
+		while( $t_row = db_fetch_array( $t_result ) ) {
+			$g_cache_access_matrix[(int)$t_row['user_id']][(int)$p_project_id] = (int)$t_row['access_level'];
 		}
 
-		$g_cache_access_matrix_project_ids[] = (int) $p_project_id;
+		$g_cache_access_matrix_project_ids[] = (int)$p_project_id;
 	}
 
 	$t_results = array();
 
 	foreach( $g_cache_access_matrix as $t_user ) {
-		if( isset( $t_user[(int) $p_project_id] ) ) {
-			$t_results[(int) $p_project_id] = $t_user[(int) $p_project_id];
+		if( isset( $t_user[(int)$p_project_id] ) ) {
+			$t_results[(int)$p_project_id] = $t_user[(int)$p_project_id];
 		}
 	}
 
@@ -159,35 +147,29 @@ function access_cache_matrix_project( $p_project_id ) {
 
 /**
  * retrieves and returns access matrix for a user from cache or caching if required.
- * @param int $p_user_id integer representing user id
- * @return  array returns an array of projects->accesslevel for the given user
+ * @param integer $p_user_id Integer representing user identifier.
+ * @return array returns an array of projects->accesslevel for the given user
  * @access private
  */
 function access_cache_matrix_user( $p_user_id ) {
 	global $g_cache_access_matrix, $g_cache_access_matrix_user_ids;
 
-	if( !in_array( (int) $p_user_id, $g_cache_access_matrix_user_ids ) ) {
-		$t_project_user_list_table = db_get_table( 'mantis_project_user_list_table' );
-
-		$query = "SELECT project_id, access_level
-					  FROM $t_project_user_list_table
-					  WHERE user_id=" . db_param();
-		$result = db_query_bound( $query, Array( (int)$p_user_id ) );
-
-		$count = db_num_rows( $result );
+	if( !in_array( (int)$p_user_id, $g_cache_access_matrix_user_ids ) ) {
+		db_param_push();
+		$t_query = 'SELECT project_id, access_level FROM {project_user_list} WHERE user_id=' . db_param();
+		$t_result = db_query( $t_query, array( (int)$p_user_id ) );
 
 		# make sure we always have an array to return
-		$g_cache_access_matrix[(int) $p_user_id] = array();
+		$g_cache_access_matrix[(int)$p_user_id] = array();
 
-		for( $i = 0;$i < $count;$i++ ) {
-			$row = db_fetch_array( $result );
-			$g_cache_access_matrix[(int) $p_user_id][(int) $row['project_id']] = (int) $row['access_level'];
+		while( $t_row = db_fetch_array( $t_result ) ) {
+			$g_cache_access_matrix[(int)$p_user_id][(int)$t_row['project_id']] = (int)$t_row['access_level'];
 		}
 
-		$g_cache_access_matrix_user_ids[] = (int) $p_user_id;
+		$g_cache_access_matrix_user_ids[] = (int)$p_user_id;
 	}
 
-	return $g_cache_access_matrix[(int) $p_user_id];
+	return $g_cache_access_matrix[(int)$p_user_id];
 }
 
 /**
@@ -196,9 +178,9 @@ function access_cache_matrix_user( $p_user_id ) {
  * $p_threshold may be a single value, or an array. If it is a single
  * value, treat it as a threshold so return true if user is >= threshold.
  * If it is an array, look for exact matches to one of the values
- * @param int $p_user_access_level user access level
- * @param int|array $p_threshold access threshold, defaults to NOBODY
- * @return bool true or false depending on whether given access level matches the threshold
+ * @param integer       $p_user_access_level User access level.
+ * @param integer|array $p_threshold         Access threshold, defaults to NOBODY.
+ * @return boolean true or false depending on whether given access level matches the threshold
  * @access public
  */
 function access_compare_level( $p_user_access_level, $p_threshold = NOBODY ) {
@@ -212,8 +194,8 @@ function access_compare_level( $p_user_access_level, $p_threshold = NOBODY ) {
 /**
  * This function only checks the user's global access level, ignoring any
  * overrides they might have at a project level
- * @param int|null $p_user_id integer representing user id, defaults to null to use current user
- * @return int global access level
+ * @param integer|null $p_user_id Integer representing user identifier, defaults to null to use current user.
+ * @return integer global access level
  * @access public
  */
 function access_get_global_level( $p_user_id = null ) {
@@ -234,9 +216,9 @@ function access_get_global_level( $p_user_id = null ) {
 /**
  * Check the current user's access against the given value and return true
  * if the user's access is equal to or higher, false otherwise.
- * @param int $p_access_level integer representing access level
- * @param int|null $p_user_id integer representing user id, defaults to null to use current user
- * @return bool whether user has access level specified
+ * @param integer      $p_access_level Integer representing access level.
+ * @param integer|null $p_user_id      Integer representing user id, defaults to null to use current user.
+ * @return boolean whether user has access level specified
  * @access public
  */
 function access_has_global_level( $p_access_level, $p_user_id = null ) {
@@ -258,9 +240,10 @@ function access_has_global_level( $p_access_level, $p_user_id = null ) {
  * Check if the user has the specified global access level
  * and deny access to the page if not
  * @see access_has_global_level
- * @param int $p_access_level integer representing access level
- * @param int|null $p_user_id integer representing user id, defaults to null to use current user
+ * @param integer      $p_access_level Integer representing access level.
+ * @param integer|null $p_user_id      Integer representing user id, defaults to null to use current user.
  * @access public
+ * @return void
  */
 function access_ensure_global_level( $p_access_level, $p_user_id = null ) {
 	if( !access_has_global_level( $p_access_level, $p_user_id ) ) {
@@ -272,9 +255,9 @@ function access_ensure_global_level( $p_access_level, $p_user_id = null ) {
  * This function checks the project access level first (for the current project
  * if none is specified) and if the user is not listed, it falls back on the
  * user's global access level.
- * @param int $p_project_id integer representing project id to check access against
- * @param int|null $p_user_id integer representing user id, defaults to null to use current user
- * @return int access level user has to given project
+ * @param integer      $p_project_id Integer representing project id to check access against.
+ * @param integer|null $p_user_id    Integer representing user id, defaults to null to use current user.
+ * @return integer access level user has to given project
  * @access public
  */
 function access_get_project_level( $p_project_id = null, $p_user_id = null ) {
@@ -283,7 +266,7 @@ function access_get_project_level( $p_project_id = null, $p_user_id = null ) {
 	}
 
 	# Deal with not logged in silently in this case
-	/** @todo we may be able to remove this and just error and once we default to anon login, we can remove it for sure */
+	# @todo we may be able to remove this and just error and once we default to anon login, we can remove it for sure
 	if( empty( $p_user_id ) && !auth_is_user_authenticated() ) {
 		return ANYBODY;
 	}
@@ -326,10 +309,10 @@ function access_get_project_level( $p_project_id = null, $p_user_id = null ) {
 /**
  * Check the current user's access against the given value and return true
  * if the user's access is equal to or higher, false otherwise.
- * @param int $p_access_level integer representing access level
- * @param int $p_project_id integer representing project id to check access against
- * @param int|null $p_user_id integer representing user id, defaults to null to use current user
- * @return bool whether user has access level specified
+ * @param integer      $p_access_level Integer representing access level.
+ * @param integer      $p_project_id   Integer representing project id to check access against.
+ * @param integer|null $p_user_id      Integer representing user id, defaults to null to use current user.
+ * @return boolean whether user has access level specified
  * @access public
  */
 function access_has_project_level( $p_access_level, $p_project_id = null, $p_user_id = null ) {
@@ -354,10 +337,11 @@ function access_has_project_level( $p_access_level, $p_project_id = null, $p_use
  * Check if the user has the specified access level for the given project
  * and deny access to the page if not
  * @see access_has_project_level
- * @param int $p_access_level integer representing access level
- * @param int|null $p_project_id integer representing project id to check access against, defaults to null to use current project
- * @param int|null $p_user_id integer representing user id, defaults to null to use current user
+ * @param integer      $p_access_level Integer representing access level.
+ * @param integer|null $p_project_id   Integer representing project id to check access against, defaults to null to use current project.
+ * @param integer|null $p_user_id      Integer representing user identifier, defaults to null to use current user.
  * @access public
+ * @return void
  */
 function access_ensure_project_level( $p_access_level, $p_project_id = null, $p_user_id = null ) {
 	if( !access_has_project_level( $p_access_level, $p_project_id, $p_user_id ) ) {
@@ -367,14 +351,13 @@ function access_ensure_project_level( $p_access_level, $p_project_id = null, $p_
 
 /**
  * Check whether the user has the specified access level for any project project
- * @param int $p_access_level integer representing access level
- * @param int|null $p_user_id integer representing user id, defaults to null to use current user
- * @return bool whether user has access level specified
+ * @param integer      $p_access_level Integer representing access level.
+ * @param integer|null $p_user_id      Integer representing user id, defaults to null to use current user.
+ * @return boolean whether user has access level specified
  * @access public
  */
 function access_has_any_project( $p_access_level, $p_user_id = null ) {
 	# Short circuit the check in this case
-
 	if( NOBODY == $p_access_level ) {
 		return false;
 	}
@@ -385,7 +368,7 @@ function access_has_any_project( $p_access_level, $p_user_id = null ) {
 
 	$t_projects = project_get_all_rows();
 	foreach( $t_projects as $t_project ) {
-		if ( access_has_project_level( $p_access_level, $t_project['id'], $p_user_id ) ) {
+		if( access_has_project_level( $p_access_level, $t_project['id'], $p_user_id ) ) {
 			return true;
 		}
 	}
@@ -398,10 +381,10 @@ function access_has_any_project( $p_access_level, $p_user_id = null ) {
  * if the user's access is equal to or higher, false otherwise.
  * This function looks up the bug's project and performs an access check
  * against that project
- * @param int $p_access_level integer representing access level
- * @param int $p_bug_id integer representing bug id to check access against
- * @param int|null $p_user_id integer representing user id, defaults to null to use current user
- * @return bool whether user has access level specified
+ * @param integer      $p_access_level Integer representing access level.
+ * @param integer      $p_bug_id       Integer representing bug id to check access against.
+ * @param integer|null $p_user_id      Integer representing user id, defaults to null to use current user.
+ * @return boolean whether user has access level specified
  * @access public
  */
 function access_has_bug_level( $p_access_level, $p_bug_id, $p_user_id = null ) {
@@ -433,7 +416,7 @@ function access_has_bug_level( $p_access_level, $p_bug_id, $p_user_id = null ) {
 			$t_report_bug_threshold = config_get( 'report_bug_threshold', null, $p_user_id, $t_project_id );
 			if( !is_array( $t_report_bug_threshold ) ) {
 				$s_thresholds[$t_project_id] = $t_report_bug_threshold + 1;
-			} else if ( empty( $t_report_bug_threshold ) ) {
+			} else if( empty( $t_report_bug_threshold ) ) {
 				$s_thresholds[$t_project_id] = NOBODY;
 			} else {
 				sort( $t_report_bug_threshold );
@@ -456,14 +439,33 @@ function access_has_bug_level( $p_access_level, $p_bug_id, $p_user_id = null ) {
 	return access_compare_level( $t_access_level, $p_access_level );
 }
 
+/**
+ * Filter the provided array of user ids to those who has the specified access level for the
+ * specified bug.
+ * @param  int $p_access_level   The access level.
+ * @param  int $p_bug_id         The bug id.
+ * @param  array $p_user_ids     The array of user ids.
+ * @return array filtered array of user ids.
+ */
+function access_has_bug_level_filter( $p_access_level, $p_bug_id, $p_user_ids ) {
+	$t_users_ids_with_access = array();
+	foreach( $p_user_ids as $t_user_id ) {
+		if( access_has_bug_level( $p_access_level, $p_bug_id, $t_user_id ) ) {
+			$t_users_ids_with_access[] = $t_user_id;
+		}
+	}
+
+	return $t_users_ids_with_access;
+}
+
 /**
  * Check if the user has the specified access level for the given bug
  * and deny access to the page if not
  * @see access_has_bug_level
- * @param int $p_access_level integer representing access level
- * @param int $p_bug_id integer representing bug id to check access against
- * @param int|null $p_user_id integer representing user id, defaults to null to use current user
- * @return bool whether user has access level specified
+ * @param integer      $p_access_level Integer representing access level.
+ * @param integer      $p_bug_id       Integer representing bug id to check access against.
+ * @param integer|null $p_user_id      Integer representing user id, defaults to null to use current user.
+ * @return void
  * @access public
  */
 function access_ensure_bug_level( $p_access_level, $p_bug_id, $p_user_id = null ) {
@@ -477,10 +479,10 @@ function access_ensure_bug_level( $p_access_level, $p_bug_id, $p_user_id = null
  * if the user's access is equal to or higher, false otherwise.
  * This function looks up the bugnote's bug and performs an access check
  * against that bug
- * @param int $p_access_level integer representing access level
- * @param int $p_bugnote_id integer representing bugnote id to check access against
- * @param int|null $p_user_id integer representing user id, defaults to null to use current user
- * @return bool whether user has access level specified
+ * @param integer      $p_access_level Integer representing access level.
+ * @param integer      $p_bugnote_id   Integer representing bugnote id to check access against.
+ * @param integer|null $p_user_id      Integer representing user id, defaults to null to use current user.
+ * @return boolean whether user has access level specified
  * @access public
  */
 function access_has_bugnote_level( $p_access_level, $p_bugnote_id, $p_user_id = null ) {
@@ -493,7 +495,7 @@ function access_has_bugnote_level( $p_access_level, $p_bugnote_id, $p_user_id =
 
 	# If the bug is private and the user is not the reporter, then the
 	# the user must also have higher access than private_bug_threshold
-	if ( bugnote_get_field( $p_bugnote_id, 'view_state' ) == VS_PRIVATE && !bugnote_is_user_reporter( $p_bugnote_id, $p_user_id ) ) {
+	if( bugnote_get_field( $p_bugnote_id, 'view_state' ) == VS_PRIVATE && !bugnote_is_user_reporter( $p_bugnote_id, $p_user_id ) ) {
 		$t_private_bugnote_threshold = config_get( 'private_bugnote_threshold', null, $p_user_id, $t_project_id );
 		$p_access_level = max( $p_access_level, $t_private_bugnote_threshold );
 	}
@@ -501,14 +503,34 @@ function access_has_bugnote_level( $p_access_level, $p_bugnote_id, $p_user_id =
 	return access_has_bug_level( $p_access_level, $t_bug_id, $p_user_id );
 }
 
+/**
+ * Filter the provided array of user ids to those who has the specified access level for the
+ * specified bugnote.
+ * @param  int $p_access_level   The access level.
+ * @param  int $p_bugnote_id     The bugnote id.
+ * @param  array $p_user_ids     The array of user ids.
+ * @return array filtered array of user ids.
+ */
+function access_has_bugnote_level_filter( $p_access_level, $p_bugnote_id, $p_user_ids ) {
+	$t_users_ids_with_access = array();
+	foreach( $p_user_ids as $t_user_id ) {
+		if( access_has_bugnote_level( $p_access_level, $p_bugnote_id, $t_user_id ) ) {
+			$t_users_ids_with_access[] = $t_user_id;
+		}
+	}
+
+	return $t_users_ids_with_access;
+}
+
 /**
  * Check if the user has the specified access level for the given bugnote
  * and deny access to the page if not
  * @see access_has_bugnote_level
- * @param int $p_access_level integer representing access level
- * @param int $p_bugnote_id integer representing bugnote id to check access against
- * @param int|null $p_user_id integer representing user id, defaults to null to use current user
+ * @param integer      $p_access_level Integer representing access level.
+ * @param integer      $p_bugnote_id   Integer representing bugnote id to check access against.
+ * @param integer|null $p_user_id      Integer representing user id, defaults to null to use current user.
  * @access public
+ * @return void
  */
 function access_ensure_bugnote_level( $p_access_level, $p_bugnote_id, $p_user_id = null ) {
 	if( !access_has_bugnote_level( $p_access_level, $p_bugnote_id, $p_user_id ) ) {
@@ -518,12 +540,12 @@ function access_ensure_bugnote_level( $p_access_level, $p_bugnote_id, $p_user_id
 
 /**
  * Check if the specified bug can be closed
- * @param BugData $p_bug Bug to check access against
- * @param int|null $p_user_id integer representing user id, defaults to null to use current user
- * @return bool true if user can close the bug
+ * @param BugData      $p_bug     Bug to check access against.
+ * @param integer|null $p_user_id Integer representing user id, defaults to null to use current user.
+ * @return boolean true if user can close the bug
  * @access public
  */
-function access_can_close_bug( $p_bug, $p_user_id = null ) {
+function access_can_close_bug( BugData $p_bug, $p_user_id = null ) {
 	if( bug_is_closed( $p_bug->id ) ) {
 		# Can't close a bug that's already closed
 		return false;
@@ -535,9 +557,8 @@ function access_can_close_bug( $p_bug, $p_user_id = null ) {
 
 	# If allow_reporter_close is enabled, then reporters can close their own bugs
 	# if they are in resolved status
-	if(    ON == config_get( 'allow_reporter_close', null, null, $p_bug->project_id )
+	if( ON == config_get( 'allow_reporter_close', null, null, $p_bug->project_id )
 		&& bug_is_user_reporter( $p_bug->id, $p_user_id )
-		&& access_has_bug_level( config_get( 'report_bug_threshold' ), $p_bug->id, $p_user_id )
 		&& bug_is_resolved( $p_bug->id )
 	) {
 		return true;
@@ -551,11 +572,12 @@ function access_can_close_bug( $p_bug, $p_user_id = null ) {
 /**
  * Make sure that the user can close the specified bug
  * @see access_can_close_bug
- * @param BugData $p_bug Bug to check access against
- * @param int|null $p_user_id integer representing user id, defaults to null to use current user
+ * @param BugData      $p_bug     Bug to check access against.
+ * @param integer|null $p_user_id Integer representing user id, defaults to null to use current user.
  * @access public
+ * @return void
  */
-function access_ensure_can_close_bug( $p_bug, $p_user_id = null ) {
+function access_ensure_can_close_bug( BugData $p_bug, $p_user_id = null ) {
 	if( !access_can_close_bug( $p_bug, $p_user_id ) ) {
 		access_denied();
 	}
@@ -563,12 +585,12 @@ function access_ensure_can_close_bug( $p_bug, $p_user_id = null ) {
 
 /**
  * Check if the specified bug can be reopened
- * @param BugData $p_bug Bug to check access against
- * @param int|null $p_user_id integer representing user id, defaults to null to use current user
- * @return bool whether user has access to reopen bugs
+ * @param BugData      $p_bug     Bug to check access against.
+ * @param integer|null $p_user_id Integer representing user id, defaults to null to use current user.
+ * @return boolean whether user has access to reopen bugs
  * @access public
  */
-function access_can_reopen_bug( $p_bug, $p_user_id = null ) {
+function access_can_reopen_bug( BugData $p_bug, $p_user_id = null ) {
 	if( !bug_is_resolved( $p_bug->id ) ) {
 		# Can't reopen a bug that's not resolved
 		return false;
@@ -578,9 +600,16 @@ function access_can_reopen_bug( $p_bug, $p_user_id = null ) {
 		$p_user_id = auth_get_current_user_id();
 	}
 
+	$t_reopen_status = config_get( 'bug_reopen_status', null, null, $p_bug->project_id );
+
+	# Reopen status must be reachable by workflow
+	if( !bug_check_workflow( $p_bug->status, $t_reopen_status ) ) {
+		return false;
+	}
+
 	# If allow_reporter_reopen is enabled, then reporters can always reopen
 	# their own bugs as long as their access level is reporter or above
-	if(    ON == config_get( 'allow_reporter_reopen', null, null, $p_bug->project_id )
+	if( ON == config_get( 'allow_reporter_reopen', null, null, $p_bug->project_id )
 		&& bug_is_user_reporter( $p_bug->id, $p_user_id )
 		&& access_has_project_level( config_get( 'report_bug_threshold', null, $p_user_id, $p_bug->project_id ), $p_bug->project_id, $p_user_id )
 	) {
@@ -590,7 +619,6 @@ function access_can_reopen_bug( $p_bug, $p_user_id = null ) {
 	# Other users's access level must allow them to reopen bugs
 	$t_reopen_bug_threshold = config_get( 'reopen_bug_threshold', null, null, $p_bug->project_id );
 	if( access_has_bug_level( $t_reopen_bug_threshold, $p_bug->id, $p_user_id ) ) {
-		$t_reopen_status = config_get( 'bug_reopen_status', null, null, $p_bug->project_id );
 
 		# User must be allowed to change status to reopen status
 		$t_reopen_status_threshold = access_get_status_threshold( $t_reopen_status, $p_bug->project_id );
@@ -604,11 +632,12 @@ function access_can_reopen_bug( $p_bug, $p_user_id = null ) {
  * Make sure that the user can reopen the specified bug.
  * Calls access_denied if user has no access to terminate script
  * @see access_can_reopen_bug
- * @param BugData $p_bug Bug to check access against
- * @param int|null $p_user_id integer representing user id, defaults to null to use current user
+ * @param BugData      $p_bug     Bug to check access against.
+ * @param integer|null $p_user_id Integer representing user id, defaults to null to use current user.
  * @access public
+ * @return void
  */
-function access_ensure_can_reopen_bug( $p_bug, $p_user_id = null ) {
+function access_ensure_can_reopen_bug( BugData $p_bug, $p_user_id = null ) {
 	if( !access_can_reopen_bug( $p_bug, $p_user_id ) ) {
 		access_denied();
 	}
@@ -617,16 +646,16 @@ function access_ensure_can_reopen_bug( $p_bug, $p_user_id = null ) {
 /**
  * get the user's access level specific to this project.
  * return false (0) if the user has no access override here
- * @param int $p_user_id Integer representing user id
- * @param int $p_project_id integer representing project id
- * @return bool|int returns false (if no access) or an integer representing level of access
+ * @param integer $p_user_id    Integer representing user id.
+ * @param integer $p_project_id Integer representing project id.
+ * @return boolean|integer returns false (if no access) or an integer representing level of access
  * @access public
  */
 function access_get_local_level( $p_user_id, $p_project_id ) {
 	global $g_cache_access_matrix, $g_cache_access_matrix_project_ids;
 
-	$p_project_id = (int) $p_project_id;
-	$p_user_id = (int) $p_user_id;
+	$p_project_id = (int)$p_project_id;
+	$p_user_id = (int)$p_user_id;
 
 	if( in_array( $p_project_id, $g_cache_access_matrix_project_ids ) ) {
 		if( isset( $g_cache_access_matrix[$p_user_id][$p_project_id] ) ) {
@@ -649,9 +678,9 @@ function access_get_local_level( $p_user_id, $p_project_id ) {
  * get the access level required to change the issue to the new status
  * If there is no specific differentiated access level, use the
  * generic update_bug_status_threshold.
- * @param int $p_status
- * @param int $p_project_id Default value ALL_PROJECTS
- * @return int integer representing user level e.g. DEVELOPER
+ * @param integer $p_status     Status.
+ * @param integer $p_project_id Default value ALL_PROJECTS.
+ * @return integer integer representing user level e.g. DEVELOPER
  * @access public
  */
 function access_get_status_threshold( $p_status, $p_project_id = ALL_PROJECTS ) {
@@ -666,3 +695,17 @@ function access_get_status_threshold( $p_status, $p_project_id = ALL_PROJECTS )
 		}
 	}
 }
+
+/**
+ * Given a access level, return the appropriate string for it
+ * @param integer $p_access_level
+ * @return string
+ */
+function access_level_get_string( $p_access_level ) {
+	if( $p_access_level > ANYBODY ) {
+		$t_access_level_string = get_enum_element( 'access_levels', $p_access_level );
+	} else {
+		$t_access_level_string = lang_get( 'no_access' );
+	}
+	return $t_access_level_string;
+}
