@@ -1,5 +1,5 @@
 <?php
-# MantisBT - a php based bugtracking system
+# MantisBT - A PHP based bugtracking system
 
 # MantisBT is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,22 +17,35 @@
 
 /**
  * Plugin API
+ *
  * Handles the initialisation, management, and execution of plugins.
+ *
  * @package CoreAPI
  * @subpackage PluginAPI
- * @copyright Copyright (C) 2000 - 2002  Kenzaburo Ito - kenito@300baud.org
- * @copyright Copyright (C) 2002 - 2014  MantisBT Team - mantisbt-dev@lists.sourceforge.net
+ * @copyright Copyright 2000 - 2002  Kenzaburo Ito - kenito@300baud.org
+ * @copyright Copyright 2002  MantisBT Team - mantisbt-dev@lists.sourceforge.net
  * @link http://www.mantisbt.org
+ *
+ * @uses access_api.php
+ * @uses config_api.php
+ * @uses constant_inc.php
+ * @uses database_api.php
+ * @uses error_api.php
+ * @uses event_api.php
+ * @uses helper_api.php
+ * @uses history_api.php
+ * @uses lang_api.php
  */
 
-$t_class_path = config_get_global( 'class_path' );
-
-/**
- * requires MantisPlugin.class
- */
-require_once( $t_class_path . 'MantisPlugin.class.php' );
-
-unset( $t_class_path );
+require_api( 'access_api.php' );
+require_api( 'config_api.php' );
+require_api( 'constant_inc.php' );
+require_api( 'database_api.php' );
+require_api( 'error_api.php' );
+require_api( 'event_api.php' );
+require_api( 'helper_api.php' );
+require_api( 'history_api.php' );
+require_api( 'lang_api.php' );
 
 # Cache variables #####
 
@@ -53,11 +66,12 @@ function plugin_get_current() {
 
 /**
  * Add the current plugin to the stack
- * @param string Plugin basename
+ * @param string $p_base_name Plugin base name.
+ * @return void
  */
-function plugin_push_current( $p_basename ) {
+function plugin_push_current( $p_base_name ) {
 	global $g_plugin_current;
-	array_unshift( $g_plugin_current, $p_basename );
+	array_unshift( $g_plugin_current, $p_base_name );
 }
 
 /**
@@ -70,16 +84,55 @@ function plugin_pop_current() {
 }
 
 /**
- * Get the URL to the plugin wrapper page.
- * @param string Page name
- * @param string Plugin basename (defaults to current plugin)
+ * Returns the list of force-installed plugins
+ * @see $g_plugins_force_installed
+ * @return array List of plugins (basename => priority)
  */
-function plugin_page( $p_page, $p_redirect = false, $p_basename = null ) {
+function plugin_get_force_installed() {
+	$t_forced_plugins = config_get_global( 'plugins_force_installed' );
+
+	# MantisCore pseudo-plugin is force-installed by definition, with priority 3
+	$t_forced_plugins['MantisCore'] = 3;
+
+	return $t_forced_plugins;
+}
+
+/**
+ * Returns an object representing the specified plugin
+ * Triggers an error if the plugin is not registered
+ * @param string|null $p_basename Plugin base name (defaults to current plugin).
+ * @return object Plugin Object
+ */
+function plugin_get( $p_basename = null ) {
+	global $g_plugin_cache;
+
 	if( is_null( $p_basename ) ) {
 		$t_current = plugin_get_current();
 	} else {
 		$t_current = $p_basename;
 	}
+
+	if( !plugin_is_registered( $t_current ) ) {
+		error_parameters( $t_current );
+		trigger_error( ERROR_PLUGIN_NOT_REGISTERED, ERROR );
+	}
+
+	return $g_plugin_cache[$p_basename];
+}
+
+/**
+ * Get the URL to the plugin wrapper page.
+ * @param string  $p_page      Page name.
+ * @param boolean $p_redirect  Return url for redirection.
+ * @param string  $p_base_name Plugin base name (defaults to current plugin).
+ * @return string
+ */
+function plugin_page( $p_page, $p_redirect = false, $p_base_name = null ) {
+	if( is_null( $p_base_name ) ) {
+		$t_current = plugin_get_current();
+	} else {
+		$t_current = $p_base_name;
+	}
 	if( $p_redirect ) {
 		return 'plugin.php?page=' . $t_current . '/' . $p_page;
 	} else {
@@ -89,28 +142,30 @@ function plugin_page( $p_page, $p_redirect = false, $p_basename = null ) {
 
 /**
  * Return a path to a plugin file.
- * @param string File name
- * @param string Plugin basename
+ * @param string $p_filename  File name.
+ * @param string $p_base_name Plugin base name.
  * @return mixed File path or false if FNF
  */
-function plugin_file_path( $p_filename, $p_basename ) {
+function plugin_file_path( $p_filename, $p_base_name ) {
 	$t_file_path = config_get( 'plugin_path' );
-	$t_file_path .= $p_basename . DIRECTORY_SEPARATOR;
+	$t_file_path .= $p_base_name . DIRECTORY_SEPARATOR;
 	$t_file_path .= 'files' . DIRECTORY_SEPARATOR . $p_filename;
 
 	return( is_file( $t_file_path ) ? $t_file_path : false );
 }
 
 /**
- * Get the URL to the plugin wrapper page.
- * @param string Page name
- * @param string Plugin basename (defaults to current plugin)
+ * Get the URL to the plugin wrapper file page.
+ * @param string  $p_file      File name.
+ * @param boolean $p_redirect  Return url for redirection.
+ * @param string  $p_base_name Plugin base name (defaults to current plugin).
+ * @return string
  */
-function plugin_file( $p_file, $p_redirect = false, $p_basename = null ) {
-	if( is_null( $p_basename ) ) {
+function plugin_file( $p_file, $p_redirect = false, $p_base_name = null ) {
+	if( is_null( $p_base_name ) ) {
 		$t_current = plugin_get_current();
 	} else {
-		$t_current = $p_basename;
+		$t_current = $p_base_name;
 	}
 	if( $p_redirect ) {
 		return 'plugin_file.php?file=' . $t_current . '/' . $p_file;
@@ -121,12 +176,12 @@ function plugin_file( $p_file, $p_redirect = false, $p_basename = null ) {
 
 /**
  * Include the contents of a file as output.
- * @param string File name
- * @param string Plugin basename
+ * @param string $p_filename File name.
+ * @param string $p_basename Plugin basename.
+ * @return void
  */
 function plugin_file_include( $p_filename, $p_basename = null ) {
-
-    global $g_plugin_mime_types;
+	global $g_plugin_mime_types;
 
 	if( is_null( $p_basename ) ) {
 		$t_current = plugin_get_current();
@@ -136,30 +191,32 @@ function plugin_file_include( $p_filename, $p_basename = null ) {
 
 	$t_file_path = plugin_file_path( $p_filename, $t_current );
 	if( false === $t_file_path ) {
-		trigger_error( ERROR_GENERIC, ERROR );
+		error_parameters( $t_current, $p_filename );
+		trigger_error( ERROR_PLUGIN_FILE_NOT_FOUND, ERROR );
 	}
 
 	$t_content_type = '';
-	$finfo = finfo_get_if_available();
+	$t_finfo = finfo_get_if_available();
 
-	if ( $finfo ) {
-		$t_file_info_type = $finfo->file( $t_file_path );
-		if ( $t_file_info_type !== false ) {
+	if( $t_finfo ) {
+		$t_file_info_type = $t_finfo->file( $t_file_path );
+		if( $t_file_info_type !== false ) {
 			$t_content_type = $t_file_info_type;
 		}
 	}
 
-	// allow overriding the content type for specific text and image extensions
-	// see bug #13193 for details
-	if ( strpos($t_content_type, 'text/') === 0 || strpos( $t_content_type, 'image/') === 0 ) {
+	# allow overriding the content type for specific text and image extensions
+	# see bug #13193 for details
+	if( strpos( $t_content_type, 'text/' ) === 0 || strpos( $t_content_type, 'image/' ) === 0 ) {
 		$t_extension = pathinfo( $t_file_path, PATHINFO_EXTENSION );
-		if ( $t_extension && array_key_exists( $t_extension , $g_plugin_mime_types ) ) {
-			$t_content_type =  $g_plugin_mime_types [ $t_extension ];
+		if( $t_extension && array_key_exists( $t_extension, $g_plugin_mime_types ) ) {
+			$t_content_type =  $g_plugin_mime_types[$t_extension];
 		}
 	}
 
-	if ( $t_content_type )
-    	header('Content-Type: ' . $t_content_type );
+	if( $t_content_type ) {
+		header( 'Content-Type: ' . $t_content_type );
+	}
 
 	readfile( $t_file_path );
 }
@@ -167,8 +224,8 @@ function plugin_file_include( $p_filename, $p_basename = null ) {
 /**
  * Given a base table name for a plugin, add appropriate prefix and suffix.
  * Convenience for plugin schema definitions.
- * @param string Table name
- * @param string Plugin basename (defaults to current plugin)
+ * @param string $p_name     Table name.
+ * @param string $p_basename Plugin basename (defaults to current plugin).
  * @return string Full table name
  */
 function plugin_table( $p_name, $p_basename = null ) {
@@ -177,16 +234,24 @@ function plugin_table( $p_name, $p_basename = null ) {
 	} else {
 		$t_current = $p_basename;
 	}
-	return config_get_global( 'db_table_prefix' ) . '_plugin_' . $t_current . '_' . $p_name . config_get_global( 'db_table_suffix' );
+
+	# Determine plugin table prefix including trailing '_'
+	$t_prefix = trim( config_get_global( 'db_table_plugin_prefix' ) );
+	if( !empty( $t_prefix ) && '_' != substr( $t_prefix, -1 ) ) {
+		$t_prefix .= '_';
+	}
+
+	return db_get_table( $t_prefix . $t_current . '_' . $p_name );
 }
 
 /**
  * Get a plugin configuration option.
- * @param string Configuration option name
- * @param multi Default option value
- * @param boolean Global value
- * @param int User ID
- * @param int Project ID
+ * @param string  $p_option  Configuration option name.
+ * @param mixed   $p_default Default option value.
+ * @param boolean $p_global  Get global config variables only.
+ * @param integer $p_user    A user identifier.
+ * @param integer $p_project A Project identifier.
+ * @return string
  */
 function plugin_config_get( $p_option, $p_default = null, $p_global = false, $p_user = null, $p_project = null ) {
 	$t_basename = plugin_get_current();
@@ -201,11 +266,12 @@ function plugin_config_get( $p_option, $p_default = null, $p_global = false, $p_
 
 /**
  * Set a plugin configuration option in the database.
- * @param string Configuration option name
- * @param multi Option value
- * @param int User ID
- * @param int Project ID
- * @param int Access threshold
+ * @param string  $p_option  Configuration option name.
+ * @param mixed   $p_value   Option value.
+ * @param integer $p_user    A user identifier.
+ * @param integer $p_project A project identifier.
+ * @param integer $p_access  Access threshold.
+ * @return void
  */
 function plugin_config_set( $p_option, $p_value, $p_user = NO_USER, $p_project = ALL_PROJECTS, $p_access = DEFAULT_ACCESS_LEVEL ) {
 	if( $p_access == DEFAULT_ACCESS_LEVEL ) {
@@ -220,9 +286,10 @@ function plugin_config_set( $p_option, $p_value, $p_user = NO_USER, $p_project =
 
 /**
  * Delete a plugin configuration option from the database.
- * @param string Configuration option name
- * @param int User ID
- * @param int Project ID
+ * @param string  $p_option  Configuration option name.
+ * @param integer $p_user    A user identifier.
+ * @param integer $p_project A project identifier.
+ * @return void
  */
 function plugin_config_delete( $p_option, $p_user = ALL_USERS, $p_project = ALL_PROJECTS ) {
 	$t_basename = plugin_get_current();
@@ -233,9 +300,10 @@ function plugin_config_delete( $p_option, $p_user = ALL_USERS, $p_project = ALL_
 
 /**
  * Set plugin default values to global values without overriding anything.
- * @param array Array of configuration option name/value pairs.
+ * @param array $p_options Array of configuration option name/value pairs.
+ * @return void
  */
-function plugin_config_defaults( $p_options ) {
+function plugin_config_defaults( array $p_options ) {
 	if( !is_array( $p_options ) ) {
 		return;
 	}
@@ -253,8 +321,8 @@ function plugin_config_defaults( $p_options ) {
 /**
  * Get a language string for the plugin.
  * Automatically prepends plugin_<basename> to the string requested.
- * @param string Language string name
- * @param string Plugin basename
+ * @param string $p_name     Language string name.
+ * @param string $p_basename Plugin basename.
  * @return string Language string
  */
 function plugin_lang_get( $p_name, $p_basename = null ) {
@@ -272,6 +340,16 @@ function plugin_lang_get( $p_name, $p_basename = null ) {
 	return $t_string;
 }
 
+/**
+ * log history event from plugin
+ * @param integer $p_bug_id     A bug identifier.
+ * @param string  $p_field_name A field name.
+ * @param string  $p_old_value  The old value.
+ * @param string  $p_new_value  The new value.
+ * @param integer $p_user_id    A user identifier.
+ * @param string  $p_basename   The plugin basename (or current plugin if null).
+ * @return void
+ */
 function plugin_history_log( $p_bug_id, $p_field_name, $p_old_value, $p_new_value = '', $p_user_id = null, $p_basename = null ) {
 	if( is_null( $p_basename ) ) {
 		$t_basename = plugin_get_current();
@@ -286,9 +364,10 @@ function plugin_history_log( $p_bug_id, $p_field_name, $p_old_value, $p_new_valu
 
 /**
  * Trigger a plugin-specific error with the given name and type.
- * @param string Error name
- * @param int Error type
- * @param string Plugin basename
+ * @param string  $p_error_name Error name.
+ * @param integer $p_error_type Error type.
+ * @param string  $p_basename   The plugin basename (or current plugin if null).
+ * @return void
  */
 function plugin_error( $p_error_name, $p_error_type = ERROR, $p_basename = null ) {
 	if( is_null( $p_basename ) ) {
@@ -298,22 +377,15 @@ function plugin_error( $p_error_name, $p_error_type = ERROR, $p_basename = null
 	}
 
 	$t_error_code = "plugin_${t_basename}_${p_error_name}";
-	$MANTIS_ERROR = lang_get( 'MANTIS_ERROR' );
 
-	if( isset( $MANTIS_ERROR[$t_error_code] ) ) {
-		trigger_error( $t_error_code, $p_error_type );
-	} else {
-		error_parameters( $p_error_name, $t_basename );
-		trigger_error( ERROR_PLUGIN_GENERIC, ERROR );
-	}
-
-	return null;
+	trigger_error( $t_error_code, $p_error_type );
 }
 
 /**
  * Hook a plugin's callback function to an event.
- * @param string Event name
- * @param string Callback function
+ * @param string $p_name     Event name.
+ * @param string $p_callback Callback function.
+ * @return void
  */
 function plugin_event_hook( $p_name, $p_callback ) {
 	$t_basename = plugin_get_current();
@@ -322,9 +394,10 @@ function plugin_event_hook( $p_name, $p_callback ) {
 
 /**
  * Hook multiple plugin callbacks at once.
- * @param array Array of event name/callback key/value pairs
+ * @param array $p_hooks Array of event name/callback key/value pairs.
+ * @return void
  */
-function plugin_event_hook_many( $p_hooks ) {
+function plugin_event_hook_many( array $p_hooks ) {
 	if( !is_array( $p_hooks ) ) {
 		return;
 	}
@@ -346,12 +419,13 @@ function plugin_event_hook_many( $p_hooks ) {
 /**
  * Allows a plugin to declare a 'child plugin' that
  * can be loaded from the same parent directory.
- * @param string Child plugin basename.
+ * @param string $p_child Child plugin basename.
+ * @return mixed
  */
 function plugin_child( $p_child ) {
-	$t_basename = plugin_get_current();
+	$t_base_name = plugin_get_current();
 
-	$t_plugin = plugin_register( $t_basename, false, $p_child );
+	$t_plugin = plugin_register( $t_base_name, false, $p_child );
 
 	if( !is_null( $t_plugin ) ) {
 		plugin_init( $p_child );
@@ -360,24 +434,22 @@ function plugin_child( $p_child ) {
 	return $t_plugin;
 }
 
-# ## Plugin Management Helpers
-
 /**
  * Checks if a given plugin has been registered and initialized,
  * and returns a boolean value representing the "loaded" state.
- * @param string Plugin basename
+ * @param string $p_base_name Plugin basename.
  * @return boolean Plugin loaded
  */
-function plugin_is_loaded( $p_basename ) {
+function plugin_is_loaded( $p_base_name ) {
 	global $g_plugin_cache_init;
 
-	return ( isset( $g_plugin_cache_init[ $p_basename ] ) && $g_plugin_cache_init[ $p_basename ] );
+	return ( isset( $g_plugin_cache_init[$p_base_name] ) && $g_plugin_cache_init[$p_base_name] );
 }
 
 /**
  * Converts a version string to an array, using some punctuation and
  * number/lettor boundaries as splitting points.
- * @param string Version string
+ * @param string $p_version Version string.
  * @return array Version array
  */
 function plugin_version_array( $p_version ) {
@@ -403,12 +475,12 @@ function plugin_version_array( $p_version ) {
 
 /**
  * Checks two version arrays sequentially for minimum or maximum version dependencies.
- * @param array Version array to check
- * @param array Version array required
- * @param boolean Minimum (false) or maximum (true) version check
- * @return int 1 if the version dependency succeeds, -1 if it fails
+ * @param array   $p_version1 Version array to check.
+ * @param array   $p_version2 Version array required.
+ * @param boolean $p_maximum  Minimum (false) or maximum (true) version check.
+ * @return integer 1 if the version dependency succeeds, -1 if it fails
  */
-function plugin_version_check( $p_version1, $p_version2, $p_maximum = false ) {
+function plugin_version_check( array $p_version1, array $p_version2, $p_maximum = false ) {
 	while( count( $p_version1 ) > 0 && count( $p_version2 ) > 0 ) {
 
 		# Grab the next version bits
@@ -417,42 +489,40 @@ function plugin_version_check( $p_version1, $p_version2, $p_maximum = false ) {
 
 		# Convert to integers if possible
 		if( is_numeric( $t_version1 ) ) {
-			$t_version1 = (int) $t_version1;
+			$t_version1 = (int)$t_version1;
 		}
 		if( is_numeric( $t_version2 ) ) {
-			$t_version2 = (int) $t_version2;
+			$t_version2 = (int)$t_version2;
 		}
 
 		# Check for immediate version differences
 		if( $p_maximum ) {
 			if( $t_version1 < $t_version2 ) {
 				return 1;
-			}
-			else if( $t_version1 > $t_version2 ) {
+			} else if( $t_version1 > $t_version2 ) {
 				return -1;
 			}
 		} else {
 			if( $t_version1 > $t_version2 ) {
 				return 1;
-			}
-			else if( $t_version1 < $t_version2 ) {
+			} else if( $t_version1 < $t_version2 ) {
 				return -1;
 			}
 		}
 	}
 
 	# Versions matched exactly
-	if ( count( $p_version1 ) == 0 && count( $p_version2 ) == 0 ) {
+	if( count( $p_version1 ) == 0 && count( $p_version2 ) == 0 ) {
 		return 1;
 	}
 
 	# Handle unmatched version bits
 	if( $p_maximum ) {
-		if ( count( $p_version2 ) > 0 ) {
+		if( count( $p_version2 ) > 0 ) {
 			return 1;
 		}
 	} else {
-		if ( count( $p_version1 ) > 0 ) {
+		if( count( $p_version1 ) > 0 ) {
 			return 1;
 		}
 	}
@@ -467,23 +537,62 @@ function plugin_version_check( $p_version1, $p_version2, $p_maximum = false ) {
  * and allows both minimum and maximum version requirements.
  * Returns 1 if plugin dependency is met, 0 if dependency not met,
  * or -1 if dependency is the wrong version.
- * @param string Plugin basename
- * @param string Required version
+ * @param string  $p_base_name   Plugin base name.
+ * @param string  $p_required    Required version.
+ * @param boolean $p_initialized Whether plugin is initialized.
  * @return integer Plugin dependency status
  */
-function plugin_dependency( $p_basename, $p_required, $p_initialized = false ) {
+function plugin_dependency( $p_base_name, $p_required, $p_initialized = false ) {
 	global $g_plugin_cache, $g_plugin_cache_init;
 
 	# check for registered dependency
-	if( isset( $g_plugin_cache[$p_basename] ) ) {
+	if( isset( $g_plugin_cache[$p_base_name] ) ) {
 
 		# require dependency initialized?
-		if( $p_initialized && !isset( $g_plugin_cache_init[$p_basename] ) ) {
+		if( $p_initialized && !isset( $g_plugin_cache_init[$p_base_name] ) ) {
 			return 0;
 		}
 
 		$t_required_array = explode( ',', $p_required );
 
+		# If the plugin's minimum dependency for MantisCore is unspecified or
+		# lower than the current release (i.e. the plugin does not specifically
+		# list the current core version as supported) and the plugin does not
+		# define a maximum dependency, we add one with the current version's
+		# minor release (i.e. for 1.3.1 we would add '<1.3').
+		# The purpose of this is to avoid compatibility issues by disabling
+		# plugins which have not been updated for a new Mantis release; authors
+		# have to revise their code (if necessary), and release a new version
+		# of the plugin with updated dependencies.
+		# To indicate compatibility (e.g. with release 1.3), one can either:
+		# 1. update the minimum required version (i.e. plugin only works with
+		#    1.3; 1.2-compatible code is maintained separately):
+		#    $this->requires = array( 'MantisCore' => '1.3' );
+		# 2. add the new release as a 2nd minimum version (i.e. the plugin is
+		#    compatible with both versions):
+		#    $this->requires = array( 'MantisCore' => '1.2, 1.3' );
+		# 3. add a maximum version higher than the new release:
+		#    $this->requires = array( 'MantisCore' => '1.2, <2.0' );
+		#    Note that this may cause the plugin to face compatibility issues
+		#    if and when a version 1.4 is released.
+		if( $p_base_name == 'MantisCore' && strpos( $p_required, '<' ) === false ) {
+			$t_version_core = substr(
+				MANTIS_VERSION,
+				0,
+				strpos( MANTIS_VERSION, '.', strpos( MANTIS_VERSION, '.' ) + 1 )
+			);
+			$t_is_current_core_supported = false;
+			foreach( $t_required_array as $t_version_required ) {
+				$t_is_current_core_supported = $t_is_current_core_supported
+					|| version_compare( trim( $t_version_required ), $t_version_core, '>=' );
+			}
+			if( !$t_is_current_core_supported ) {
+				$t_required_array[] = '<' . $t_version_core;
+			}
+		}
+
+		$t_version_installed = plugin_version_array( $g_plugin_cache[$p_base_name]->version );
+
 		foreach( $t_required_array as $t_required ) {
 			$t_required = trim( $t_required );
 			$t_maximum = false;
@@ -501,12 +610,11 @@ function plugin_dependency( $p_basename, $p_required, $p_initialized = false ) {
 				}
 			}
 
-			$t_version1 = plugin_version_array( $g_plugin_cache[$p_basename]->version );
-			$t_version2 = plugin_version_array( $t_required );
+			$t_version_required = plugin_version_array( $t_required );
 
-			$t_check = plugin_version_check( $t_version1, $t_version2, $t_maximum );
+			$t_check = plugin_version_check( $t_version_installed, $t_version_required, $t_maximum );
 
-			if ( $t_check < 1 ) {
+			if( $t_check < 1 ) {
 				return $t_check;
 			}
 		}
@@ -519,58 +627,52 @@ function plugin_dependency( $p_basename, $p_required, $p_initialized = false ) {
 
 /**
  * Checks to see if a plugin is 'protected' from uninstall.
- * @param string Plugin basename
+ * @param string $p_base_name Plugin base name.
  * @return boolean True if plugin is protected
  */
-function plugin_protected( $p_basename ) {
+function plugin_protected( $p_base_name ) {
 	global $g_plugin_cache_protected;
 
-	# For pseudo-plugin MantisCore, return protected as 1.
-	if( $p_basename == 'MantisCore' ) {
-		return 1;
-	}
-
-	return $g_plugin_cache_protected[$p_basename];
+	return $g_plugin_cache_protected[$p_base_name];
 }
 
 /**
  * Gets a plugin's priority.
- * @param string Plugin basename
+ * @param string $p_base_name Plugin base name.
  * @return int Plugin priority
  */
-function plugin_priority( $p_basename ) {
+function plugin_priority( $p_base_name ) {
 	global $g_plugin_cache_priority;
 
-	# For pseudo-plugin MantisCore, return priority as 3.
-	if( $p_basename == 'MantisCore' ) {
-		return 3;
-	}
-
-	return $g_plugin_cache_priority[$p_basename];
+	return $g_plugin_cache_priority[$p_base_name];
 }
 
-# ## Plugin management functions
 /**
  * Determine if a given plugin is installed.
- * @param string Plugin basename
+ * @param string $p_basename Plugin basename.
  * @return boolean True if plugin is installed
  */
 function plugin_is_installed( $p_basename ) {
-	$t_plugin_table = db_get_table( 'mantis_plugin_table' );
+	foreach( plugin_get_force_installed() as $t_basename => $t_priority ) {
+		if( $t_basename == $p_basename ) {
+			return true;
+		}
+	}
 
-	$t_query = "SELECT COUNT(*) FROM $t_plugin_table WHERE basename=" . db_param();
-	$t_result = db_query_bound( $t_query, array( $p_basename ) );
+	db_param_push();
+	$t_query = 'SELECT COUNT(*) FROM {plugin} WHERE basename=' . db_param();
+	$t_result = db_query( $t_query, array( $p_basename ) );
 	return( 0 < db_result( $t_result ) );
 }
 
 /**
  * Install a plugin to the database.
- * @param string Plugin basename
+ * @param MantisPlugin $p_plugin Plugin basename.
+ * @return null
  */
-function plugin_install( $p_plugin ) {
-	access_ensure_global_level( config_get_global( 'manage_plugin_threshold' ) );
-
+function plugin_install( MantisPlugin $p_plugin ) {
 	if( plugin_is_installed( $p_plugin->basename ) ) {
+		error_parameters( $p_plugin->basename );
 		trigger_error( ERROR_PLUGIN_ALREADY_INSTALLED, WARNING );
 		return null;
 	}
@@ -578,15 +680,14 @@ function plugin_install( $p_plugin ) {
 	plugin_push_current( $p_plugin->basename );
 
 	if( !$p_plugin->install() ) {
-		plugin_pop_current( $p_plugin->basename );
+		plugin_pop_current();
 		return null;
 	}
 
-	$t_plugin_table = db_get_table( 'mantis_plugin_table' );
-
-	$t_query = "INSERT INTO $t_plugin_table ( basename, enabled )
-				VALUES ( " . db_param() . ", '1' )";
-	db_query_bound( $t_query, array( $p_plugin->basename ) );
+	db_param_push();
+	$t_query = 'INSERT INTO {plugin} ( basename, enabled )
+				VALUES ( ' . db_param() . ', ' . db_param() . ' )';
+	db_query( $t_query, array( $p_plugin->basename, true ) );
 
 	if( false === ( plugin_config_get( 'schema', false ) ) ) {
 		plugin_config_set( 'schema', -1 );
@@ -599,11 +700,13 @@ function plugin_install( $p_plugin ) {
 
 /**
  * Determine if an installed plugin needs to upgrade its schema.
- * @param string Plugin basename
- * @return boolean True if plugin needs schema ugrades.
+ * @param MantisPlugin $p_plugin Plugin basename.
+ * @return boolean True if plugin needs schema upgrades.
  */
-function plugin_needs_upgrade( $p_plugin ) {
+function plugin_needs_upgrade( MantisPlugin $p_plugin ) {
+	plugin_push_current( $p_plugin->name );
 	$t_plugin_schema = $p_plugin->schema();
+	plugin_pop_current();
 	if( is_null( $t_plugin_schema ) ) {
 		return false;
 	}
@@ -616,19 +719,19 @@ function plugin_needs_upgrade( $p_plugin ) {
 
 /**
  * Upgrade an installed plugin's schema.
- * @param string Plugin basename
- * @return multi True if upgrade completed, null if problem
+ * @param MantisPlugin $p_plugin Plugin basename.
+ * @return boolean|null True if upgrade completed, null if problem
  */
-function plugin_upgrade( $p_plugin ) {
-	access_ensure_global_level( config_get_global( 'manage_plugin_threshold' ) );
-
+function plugin_upgrade( MantisPlugin $p_plugin ) {
 	if( !plugin_is_installed( $p_plugin->basename ) ) {
-		return;
+		return null;
 	}
 
+	require_api( 'install_helper_functions_api.php' );
+
 	plugin_push_current( $p_plugin->basename );
 
-	$t_schema_version = plugin_config_get( 'schema', -1 );
+	$t_schema_version = (int)plugin_config_get( 'schema', -1 );
 	$t_schema = $p_plugin->schema();
 
 	global $g_db;
@@ -643,32 +746,39 @@ function plugin_upgrade( $p_plugin ) {
 
 		$t_target = $t_schema[$i][1][0];
 
-        if ( $t_schema[$i][0] == "UpdateFunction" ) {
-            call_user_func_array( $t_schema[$i][1], $t_schema[$i][2] );
-        } else {
-            if ( $t_schema[$i][0] == 'InsertData' ) {
-                $t_sqlarray = array(
-                    'INSERT INTO ' . $t_schema[$i][1][0] . $t_schema[$i][1][1],
-                );
-            } else if ( $t_schema[$i][0] == 'UpdateSQL' ) {
-                $t_sqlarray = array(
-                    'UPDATE ' . $t_schema[$i][1][0] . $t_schema[$i][1][1],
-                );
-            } else {
-                $t_sqlarray = call_user_func_array( Array( $t_dict, $t_schema[$i][0] ), $t_schema[$i][1] );
-            }
+		if( $t_schema[$i][0] == 'InsertData' ) {
+			$t_sqlarray = array(
+				'INSERT INTO ' . $t_schema[$i][1][0] . $t_schema[$i][1][1],
+			);
+		} else if( $t_schema[$i][0] == 'UpdateSQL' ) {
+			$t_sqlarray = array(
+				'UPDATE ' . $t_schema[$i][1][0] . $t_schema[$i][1][1],
+			);
+			$t_target = $t_schema[$i][1];
+		} else if( $t_schema[$i][0] == 'UpdateFunction' ) {
+			$t_sqlarray = false;
+			if( isset( $t_schema[$i][2] ) ) {
+				$t_status = call_user_func( 'install_' . $t_schema[$i][1], $t_schema[$i][2] );
+			} else {
+				$t_status = call_user_func( 'install_' . $t_schema[$i][1] );
+			}
+		} else {
+			$t_sqlarray = call_user_func_array( array( $t_dict, $t_schema[$i][0] ), $t_schema[$i][1] );
+		}
 
-            $t_status = $t_dict->ExecuteSQLArray( $t_sqlarray, /* continue_on_error */ false );
+		if( $t_sqlarray ) {
+			$t_status = $t_dict->ExecuteSQLArray( $t_sqlarray );
+		}
 
-            if ( 2 != $t_status ) {
-                error_parameters( $i );
-                trigger_error( ERROR_PLUGIN_UPGRADE_FAILED, ERROR );
-                return null;
-            }
-        }
+		if( 2 == $t_status ) {
+			plugin_config_set( 'schema', $i );
+		} else {
+			error_parameters( $i );
+			trigger_error( ERROR_PLUGIN_UPGRADE_FAILED, ERROR );
+			return null;
+		}
 
-        plugin_config_set( 'schema', $i );
-        $i++;
+		$i++;
 	}
 
 	plugin_pop_current();
@@ -678,19 +788,19 @@ function plugin_upgrade( $p_plugin ) {
 
 /**
  * Uninstall a plugin from the database.
- * @param string Plugin basename
+ * @param MantisPlugin $p_plugin Plugin basename.
+ * @return void
  */
-function plugin_uninstall( $p_plugin ) {
+function plugin_uninstall( MantisPlugin $p_plugin ) {
 	access_ensure_global_level( config_get_global( 'manage_plugin_threshold' ) );
 
 	if( !plugin_is_installed( $p_plugin->basename ) || plugin_protected( $p_plugin->basename ) ) {
 		return;
 	}
 
-	$t_plugin_table = db_get_table( 'mantis_plugin_table' );
-
-	$t_query = "DELETE FROM $t_plugin_table WHERE basename=" . db_param();
-	db_query_bound( $t_query, array( $p_plugin->basename ) );
+	db_param_push();
+	$t_query = 'DELETE FROM {plugin} WHERE basename=' . db_param();
+	db_query( $t_query, array( $p_plugin->basename ) );
 
 	plugin_push_current( $p_plugin->basename );
 
@@ -699,7 +809,6 @@ function plugin_uninstall( $p_plugin ) {
 	plugin_pop_current();
 }
 
-# ## Core usage only.
 /**
  * Search the plugins directory for plugins.
  * @return array Plugin basename/info key/value pairs.
@@ -711,7 +820,7 @@ function plugin_find_all() {
 	);
 
 	if( $t_dir = opendir( $t_plugin_path ) ) {
-		while(( $t_file = readdir( $t_dir ) ) !== false ) {
+		while( ( $t_file = readdir( $t_dir ) ) !== false ) {
 			if( '.' == $t_file || '..' == $t_file ) {
 				continue;
 			}
@@ -730,7 +839,9 @@ function plugin_find_all() {
 
 /**
  * Load a plugin's core class file.
- * @param string Plugin basename
+ * @param string $p_basename Plugin basename.
+ * @param string $p_child    Child filename.
+ * @return boolean
  */
 function plugin_include( $p_basename, $p_child = null ) {
 	$t_path = config_get_global( 'plugin_path' ) . $p_basename . DIRECTORY_SEPARATOR;
@@ -749,21 +860,49 @@ function plugin_include( $p_basename, $p_child = null ) {
 	return $t_included;
 }
 
+/**
+ * Allows a plugin page to require a plugin-specific API
+ * @param string $p_file     The API to be included.
+ * @param string $p_basename Plugin's basename (defaults to current plugin).
+ * @return void
+ */
+function plugin_require_api( $p_file, $p_basename = null ) {
+	if( is_null( $p_basename ) ) {
+		$t_current = plugin_get_current();
+	} else {
+		$t_current = $p_basename;
+	}
+
+	$t_path = config_get_global( 'plugin_path' ) . $t_current . '/';
+
+	require_once( $t_path . $p_file );
+}
+
+/**
+ * Determine if a given plugin is registered.
+ * @param string $p_basename Plugin basename.
+ * @return boolean True if plugin is registered
+ */
+function plugin_is_registered( $p_basename ) {
+	global $g_plugin_cache;
+
+	return isset( $g_plugin_cache[$p_basename] );
+}
+
 /**
  * Register a plugin with MantisBT.
  * The plugin class must already be loaded before calling.
- * @param string Plugin classname without 'Plugin' postfix
+ * @param string  $p_basename Plugin classname without 'Plugin' postfix.
+ * @param boolean $p_return   Return.
+ * @param string  $p_child    Child filename.
+ * @return mixed
  */
 function plugin_register( $p_basename, $p_return = false, $p_child = null ) {
 	global $g_plugin_cache;
 
 	$t_basename = is_null( $p_child ) ? $p_basename : $p_child;
 	if( !isset( $g_plugin_cache[$t_basename] ) ) {
-		if( is_null( $p_child ) ) {
-			$t_classname = $p_basename . 'Plugin';
-		} else {
-			$t_classname = $p_child . 'Plugin';
-		}
+		$t_classname = $t_basename . 'Plugin';
 
 		# Include the plugin script if the class is not already declared.
 		if( !class_exists( $t_classname ) ) {
@@ -790,6 +929,9 @@ function plugin_register( $p_basename, $p_return = false, $p_child = null ) {
 			} else {
 				$g_plugin_cache[$t_basename] = $t_plugin;
 			}
+		} else {
+			error_parameters( $t_basename, $t_classname );
+			trigger_error( ERROR_PLUGIN_CLASS_NOT_FOUND, ERROR );
 		}
 	}
 
@@ -798,29 +940,41 @@ function plugin_register( $p_basename, $p_return = false, $p_child = null ) {
 
 /**
  * Find and register all installed plugins.
+ * This includes the MantisCore pseudo-plugin.
+ * @return void
  */
 function plugin_register_installed() {
 	global $g_plugin_cache_priority, $g_plugin_cache_protected;
 
-	$t_plugin_table = db_get_table( 'mantis_plugin_table' );
+	# register plugins specified in the site configuration
+	foreach( plugin_get_force_installed() as $t_basename => $t_priority ) {
+		plugin_register( $t_basename );
+		$g_plugin_cache_priority[$t_basename] = $t_priority;
+		$g_plugin_cache_protected[$t_basename] = true;
+	}
 
-	$t_query = "SELECT basename, priority, protected FROM $t_plugin_table WHERE enabled=" . db_param() . ' ORDER BY priority DESC';
-	$t_result = db_query_bound( $t_query, Array( 1 ) );
+	# register plugins installed via the interface/database
+	db_param_push();
+	$t_query = 'SELECT basename, priority, protected FROM {plugin} WHERE enabled=' . db_param() . ' ORDER BY priority DESC';
+	$t_result = db_query( $t_query, array( true ) );
 
 	while( $t_row = db_fetch_array( $t_result ) ) {
 		$t_basename = $t_row['basename'];
-		plugin_register( $t_basename );
-		$g_plugin_cache_priority[$t_basename] = $t_row['priority'];
-		$g_plugin_cache_protected[$t_basename] = $t_row['protected'];
+		if( !plugin_is_registered( $t_basename ) ) {
+			plugin_register( $t_basename );
+			$g_plugin_cache_priority[$t_basename] = (int)$t_row['priority'];
+			$g_plugin_cache_protected[$t_basename] = (bool)$t_row['protected'];
+		}
 	}
 }
 
 /**
  * Initialize all installed plugins.
  * Post-signals EVENT_PLUGIN_INIT.
+ * @return void
  */
 function plugin_init_installed() {
-	if( OFF == config_get_global( 'plugins_enabled' ) || !db_table_exists( db_get_table( 'mantis_plugin_table' ) ) ) {
+	if( OFF == config_get_global( 'plugins_enabled' ) || !db_table_exists( db_get_table( 'plugin' ) ) ) {
 		return;
 	}
 
@@ -831,7 +985,6 @@ function plugin_init_installed() {
 	$g_plugin_cache_priority = array();
 	$g_plugin_cache_protected = array();
 
-	plugin_register( 'MantisCore' );
 	plugin_register_installed();
 
 	$t_plugins = array_keys( $g_plugin_cache );
@@ -851,15 +1004,14 @@ function plugin_init_installed() {
 		}
 
 		$t_plugins = $t_plugins_retry;
-	}
-	while( $t_continue );
+	} while( $t_continue );
 
 	event_signal( 'EVENT_PLUGIN_INIT' );
 }
 
 /**
  * Initialize a single plugin.
- * @param string Plugin basename
+ * @param string $p_basename Plugin basename.
  * @return boolean True if plugin initialized, false otherwise.
  */
 function plugin_init( $p_basename ) {
@@ -883,14 +1035,14 @@ function plugin_init( $p_basename ) {
 		# registered, but not yet initialized.
 		if( is_array( $t_plugin->uses ) ) {
 			foreach( $t_plugin->uses as $t_used => $t_version ) {
-				if ( isset( $g_plugin_cache[ $t_used ] ) && !isset( $g_plugin_cache_init[ $t_used ] ) ) {
+				if( isset( $g_plugin_cache[$t_used] ) && !isset( $g_plugin_cache_init[$t_used] ) ) {
 					return false;
 				}
 			}
 		}
 
 		# if plugin schema needs an upgrade, do not initialize
-		if ( plugin_needs_upgrade( $t_plugin ) ) {
+		if( plugin_needs_upgrade( $t_plugin ) ) {
 			return false;
 		}
 
