@@ -1,47 +1,49 @@
 <?php
 
 /**
-  V5.10 10 Nov 2009   (c) 2000-2009 John Lim (jlim#natsoft.com). All rights reserved.
-  Released under both BSD license and Lesser GPL library license. 
-  Whenever there is any discrepancy between the two licenses, 
+  @version   v5.20.4  30-Mar-2016
+  @copyright (c) 2000-2013 John Lim (jlim#natsoft.com). All rights reserved.
+  @copyright (c) 2014      Damien Regad, Mark Newnham and the ADOdb community
+  Released under both BSD license and Lesser GPL library license.
+  Whenever there is any discrepancy between the two licenses,
   the BSD license will take precedence.
-	
+
   Set tabs to 4 for best viewing.
- 
+
 */
 
 // security - hide paths
 if (!defined('ADODB_DIR')) die();
 
 class ADODB2_ibase extends ADODB_DataDict {
-	
+
 	var $databaseType = 'ibase';
 	var $seqField = false;
-	
- 	
+
+
  	function ActualType($meta)
 	{
 		switch($meta) {
 		case 'C': return 'VARCHAR';
 		case 'XL':
-		case 'X': return 'VARCHAR(4000)'; 
-		
+		case 'X': return 'VARCHAR(4000)';
+
 		case 'C2': return 'VARCHAR'; // up to 32K
 		case 'X2': return 'VARCHAR(4000)';
-		
+
 		case 'B': return 'BLOB';
-			
+
 		case 'D': return 'DATE';
 		case 'TS':
 		case 'T': return 'TIMESTAMP';
-		
+
 		case 'L': return 'SMALLINT';
 		case 'I': return 'INTEGER';
 		case 'I1': return 'SMALLINT';
 		case 'I2': return 'SMALLINT';
 		case 'I4': return 'INTEGER';
 		case 'I8': return 'INTEGER';
-		
+
 		case 'F': return 'DOUBLE PRECISION';
 		case 'N': return 'DECIMAL';
 		default:
@@ -49,20 +51,17 @@ class ADODB2_ibase extends ADODB_DataDict {
 		}
 	}
 
-	function AlterColumnSQL($tabname, $flds)
+	function AlterColumnSQL($tabname, $flds, $tableflds='', $tableoptions='')
 	{
 		if ($this->debug) ADOConnection::outp("AlterColumnSQL not supported");
 		return array();
 	}
-	
-	
-	function DropColumnSQL($tabname, $flds)
+
+
+	function DropColumnSQL($tabname, $flds, $tableflds='', $tableoptions='')
 	{
 		if ($this->debug) ADOConnection::outp("DropColumnSQL not supported");
 		return array();
 	}
-	
+
 }
-
-
-?>
